<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 Peter Benke (NTT DATA) <peter.benke@nttdata.com>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 3 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Scheduler Task Fetchfiles
 *
 * @author		Peter Benke <peter.benke@nttdata.com>
 * @package 	tx_pb_fetch_remote_files
 */
class tx_pbfetchremotefiles_scheduler_task_fetchfiles extends tx_scheduler_task {

	/**
	 * File extension to save the files temporarily
	 *
	 * @var	string
	 */	
	protected $tmpFileExtension = '.tmp_pb_fetch_remote_files';
	
	/**
	 * Gets the notification email address.
	 *
	 * @return	string	Notification email address.
	 */
	public function getNotificationEmail() {
		return $this->notificationEmail;
	}

	/**
	 * Sets the notification email address.
	 *
	 * @param	string	$notificationEmail Notification email address.
	 */
	public function setNotificationEmail($notificationEmail) {
		$this->notificationEmail = $notificationEmail;
	}
	
	/**
	 * Gets the mail mode
	 *
	 * @return	string	mail mode
	 */
	public function getMailMode() {
		return $this->mailMode;
	}

	/**
	 * Sets the mail mode
	 *
	 * @param	string	$mailMode mail mode.
	 */
	public function setMailMode($mailMode) {
		$this->mailMode = $mailMode;
	}	
	
	/**
	 * Gets the Remote url main path
	 *
	 * @return	string RemoteUrlMainPath
	 */
	public function getRemoteUrlMainPath() {
		return $this->remoteUrlMainPath;
	}

	/**
	 * Sets the Remote url main path
	 *
	 * @param	string	$remoteUrlMainPath Remote url main path
	 */
	public function setRemoteUrlMainPath($remoteUrlMainPath) {
		$this->remoteUrlMainPath = $remoteUrlMainPath;
	}

	/**
	 * Gets the Local path
	 *
	 * @return	string LocalPath
	 */
	public function getLocalPath() {
		return $this->localPath;
	}

	/**
	 * Sets the Local path
	 *
	 * @param	string	$localPath Local path
	 */
	public function setLocalPath($localPath) {
		$this->localPath = $localPath;
	}	

	/**
	 * Gets the Remote Files
	 *
	 * @return	string RemoteFiles
	 */
	public function getRemoteFiles() {
		return $this->remoteFiles;
	}

	/**
	 * Sets the Remote Files
	 *
	 * @param	string	$remoteFiles Remote Files
	 */
	public function setRemoteFiles($remoteFiles) {
		$this->remoteFiles = $remoteFiles;
	}	



	/*
	 * Execute function
	 * @param none
	 * @return void
	 */
	public function execute() {

		$remoteFiles	= explode("\n", $this->getRemoteFiles());
		// $localPath		= $_SERVER['DOCUMENT_ROOT'] . '/' . $this->getLocalPath();
		$localPath		= PATH_site . $this->getLocalPath();

		if(!$this->checkLocalFilesExist($remoteFiles, $localPath)){
			return false;
		}
		
		// Now get the files
		$result = $this->curl_download($remoteFiles, $localPath);
		
		if(!$result){
			return false;
		}
		
		return true;
		
	}
	
	
	/*
	 * Check, wheather the files already exist locally? (required)
	 * If not, a mail is sent
	 * @param array remoteFiles
	 * @param string Local path
	 * @return boolean
	 */
	private function checkLocalFilesExist($remoteFiles, $localPath){
		
		// Exist the files locally? (required)
		$localFilesOk = true;
		$mailFileList = '';

		foreach($remoteFiles as $remoteFile){
			$remoteFile = chop($remoteFile);
			if(!is_file($localPath . $remoteFile)){
				$localFilesOk = false;
				$mailFileList.= "\n" . $this->getLocalPath() . $remoteFile;
			}
		}

		if(!$localFilesOk && ($this->getMailMode() == 'all' || $this->getMailMode() == 'errors')){
			$this->mailNotice('error.localFilesDoesNotExist', $mailFileList);	
		}
		
		return $localFilesOk;
		
	}
	
	/*
	 * Sends an email
	 * @param string  mode (has to definaed in locallang.xml)
	 * @param string additionalBody
	 * @return void
	 */
	private function mailNotice($mode, $additionalBody = ''){

		$LANG = t3lib_div::makeInstance('language');
		$LANG->init($GLOBALS['BE_USER'] ? $GLOBALS['BE_USER']->uc['lang'] : 'default');
		$LANG->includeLLFile('typo3conf/ext/pb_fetch_remote_files/locallang.xml');

		$recipient	= $this->getNotificationEmail();
		$header			= 'Content-type: text/plain; charset=utf-8' . "\r\n";

		$subject	= $LANG->getLL('tasks.fetchfiles.mail.' . $mode . '.subject');
		$body			= $LANG->getLL('tasks.fetchfiles.mail.' . $mode . '.body');
		
		mail($recipient, $subject, $body . "\n" . $additionalBody, $header);
		
	}
	
	

	/*
	 * Gets files from a remote server
	 * @param array remote files
	 * @param string local path (whole path from root)
	 * @retuen boolean
	 */
	private function curl_download($remoteFiles, $localPath){
	
		$transferOk = true;
		$mailFileList = '';
		
		if (!function_exists('curl_init')){
			if($this->getMailMode() == 'all' || $this->getMailMode() == 'errors'){
				$this->mailNotice('error.curlIsNotActive');	
			}
			return false;
		}

		$remotePath = $this->getRemoteUrlMainPath();
		
		
		foreach($remoteFiles as $remoteFile){
			
			$remoteFile		= chop($remoteFile);
			$url					= $remotePath . $remoteFile;
			$localFile		= $localPath . $remoteFile;
			$tmpLocalFile	= $localFile . $this->tmpFileExtension;
			
			// Start curl
			$ch = curl_init($url);
			
			// File handling
			$fp = fopen($tmpLocalFile, 'w');
			
			// Set writing into a file
			curl_setopt($ch, CURLOPT_FILE, $fp);
			
			$data = curl_exec($ch);
			$info = curl_getinfo($ch);

			fwrite($fp, $data);
			fclose($fp);
			
			curl_close($ch);
			
			// file could not be fetched
			if($info['http_code'] == '200'){
				
				if(is_file($tmpLocalFile) && is_file($localFile)){
					copy($tmpLocalFile, $localFile);
					unlink($tmpLocalFile);
				}
				
			}else{
				$transferOk = false;
			}

			$mailFileList.= "\n" . $url;			
			unset($url, $localFile, $tmpLocalFile, $ch, $fp, $data, $info);
			
		}
		
		if($transferOk){

			if($this->getMailMode() == 'all'){
				$this->mailNotice('success.filesFetched', $mailFileList);
			}
			return true;

		} else{

			if($this->getMailMode() == 'all' || $this->getMailMode() == 'errors'){
				$this->mailNotice('error.filesCouldNotBeFetched', $mailFileList);
			}
			return false;

		}
		
	}
	
	
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/pb_fetch_remote_files/lib/scheduler/class.tx_pbfetchremotefiles_tasks_fetchfiles.php']) {
	include_once ($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/pb_fetch_remote_files/lib/scheduler/class.tx_pbfetchremotefiles_tasks_fetchfiles.php']);
}

?>