<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 Peter Benke (NTT DATA) <peter.benke@nttdata.com>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 3 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * This class provides Scheduler Additional Field plugin implementation
 *
 * @author Peter Benke <peter.benke@nttdata.com>
 * @package tx_pb_fetch_remote_files
 */
class tx_pbfetchremotefiles_scheduler_task_fetchfilesadditionalfieldprovider implements tx_scheduler_AdditionalFieldProvider {

	/**
	 * Additional fields
	 *
	 * @var	array
	 */
	protected $fields = array('notificationEmail', 'remoteUrlMainPath', 'localPath', 'remoteFiles');

	/**
	 * Field prefix. (Input fields)
	 *
	 * @var	string
	 */
	protected $fieldPrefix = 'pbfetchremotefiles';

	/**
	 * Gets additional fields to render in the form to add/edit a task
	 *
	 * @param	array	$taskInfo Values of the fields from the add/edit task form
	 * @param	tx_scheduler_Task	$task The task object being eddited. Null when adding a task!
	 * @param	tx_scheduler_Module	$schedulerModule Reference to the scheduler backend module
	 * @return	array	A two dimensional array, array('Identifier' => array('fieldId' => array('code' => '', 'label' => '', 'cshKey' => '', 'cshLabel' => ''))
	 */
	public function getAdditionalFields(array &$taskInfo, $task, tx_scheduler_Module $schedulerModule) {

		$LANG = t3lib_div::makeInstance('language');
		$LANG->init($GLOBALS['BE_USER'] ? $GLOBALS['BE_USER']->uc['lang'] : 'default');
		$LANG->includeLLFile('typo3conf/ext/pb_fetch_remote_files/locallang.xml');
		
		if ($schedulerModule->CMD == 'edit'){
			$taskInfo[$this->fieldPrefix . 'NotificationEmail']	= $task->getNotificationEmail();
			$taskInfo[$this->fieldPrefix . 'MailMode']					= $task->getMailMode();
			$taskInfo[$this->fieldPrefix . 'RemoteUrlMainPath']	= $task->getRemoteUrlMainPath();
			$taskInfo[$this->fieldPrefix . 'RemoteFiles']				= $task->getRemoteFiles();
			$taskInfo[$this->fieldPrefix . 'LocalPath']					= $task->getLocalPath();
		}

		// Input fields
		// =====================================================================================
		$fields = array('notificationEmail', 'remoteUrlMainPath', 'localPath');

		$additionalFields = array();
		
		foreach ($fields as $field) {
			$fieldName = $this->getFullFieldName($field);
			$fieldId   = 'task_' . $fieldName;
			$fieldHtml = '<input type="text" style="width:370px;" '
				. 'name="tx_scheduler[' . $fieldName . ']" '
				. 'id="' . $fieldId . '" '
				. 'value="' . htmlspecialchars($taskInfo[$fieldName]) . '" />';

			$additionalFields[$fieldId] = array(
				'code'     => $fieldHtml,
				'label'    => 'LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.label.' . $field,
				'cshKey'   => '',
				'cshLabel' => $fieldId
			);
		}
		
		// Textarea
		// =====================================================================================
		$fieldName = $this->getFullFieldName('remoteFiles');
		$fieldId   = 'task_' . $fieldName;

		$fieldHtml = '<textarea style="width:370px;resize:vertical;" '
			. 'name="tx_scheduler[' . $fieldName . ']" '
			. 'id="' . $fieldId . '">'
			. htmlspecialchars($taskInfo[$fieldName]) . '</textarea>';

		$additionalFields[$fieldId] = array(
			'code'     => $fieldHtml,
			'label'    => 'LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.label.remoteFiles',
			'cshKey'   => '',
			'cshLabel' => $fieldId
		);
		
		// Selectbox
		// =====================================================================================
		$fieldName = $this->getFullFieldName('mailMode');
		$fieldId   = 'task_' . $fieldName;
		$options = array('errors', 'all', 'never');

		$fieldHtml = '<select style="width:350px;" '
			. 'name="tx_scheduler[' . $fieldName . ']" '
			. 'id="' . $fieldId . '">';
		
		foreach ($options as $option) {
			$selected = '';
			if($taskInfo[$fieldName] == $option){
				$selected = ' selected = "selected"' ;
			}
			$fieldHtml.= '<option' . $selected . ' value="' . $option . '">'	. $LANG->getLL('tasks.fetchfiles.mailMode.option.' . $option)	. '</option>';
		}
		
		$fieldHtml.= '</select>';

		$additionalFields[$fieldId] = array(
			'code'     => $fieldHtml,
			'label'    => 'LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.label.mailMode',
			'cshKey'   => '',
			'cshLabel' => $fieldId
		);
		
		
		$arrayNewOrder = array(
			'task_pbfetchremotefilesNotificationEmail'	=> $additionalFields['task_pbfetchremotefilesNotificationEmail'],
			'task_pbfetchremotefilesMailMode'						=> $additionalFields['task_pbfetchremotefilesMailMode'],
			'task_pbfetchremotefilesRemoteUrlMainPath'	=> $additionalFields['task_pbfetchremotefilesRemoteUrlMainPath'],
			'task_pbfetchremotefilesRemoteFiles'				=> $additionalFields['task_pbfetchremotefilesRemoteFiles'],
			'task_pbfetchremotefilesLocalPath'					=> $additionalFields['task_pbfetchremotefilesLocalPath'],
		);
		
		return $arrayNewOrder;
	
	}

	/**
	 * Validates the additional fields' values
	 *
	 * @param	array	$submittedData An array containing the data submitted by the add/edit task form
	 * @param	tx_scheduler_Module	$schedulerModule Reference to the scheduler backend module
	 * @return	boolean	TRUE if validation was ok (or selected class is not relevant), FALSE otherwise
	 */
	public function validateAdditionalFields(array &$submittedData, tx_scheduler_Module $schedulerModule) {
		
		$validInput = true;
		$submittedData[$this->fieldPrefix . 'NotificationEmail']	= trim($submittedData[$this->fieldPrefix . 'NotificationEmail']);
		$submittedData[$this->fieldPrefix . 'MailMode']						= trim($submittedData[$this->fieldPrefix . 'MailMode']);
		$submittedData[$this->fieldPrefix . 'RemoteUrlMainPath']	= trim($submittedData[$this->fieldPrefix . 'RemoteUrlMainPath']);
		$submittedData[$this->fieldPrefix . 'RemoteFiles']				= trim($submittedData[$this->fieldPrefix . 'RemoteFiles']);
		$submittedData[$this->fieldPrefix . 'LocalPath']					= trim($submittedData[$this->fieldPrefix . 'LocalPath']);

		// Only check, if MailMode is filled
		if (

			(
				$submittedData[$this->fieldPrefix . 'MailMode'] == 'errors'
				||
				$submittedData[$this->fieldPrefix . 'MailMode'] == 'all'
			)
			&&
			#!empty($submittedData[$this->fieldPrefix . 'NotificationEmail'])
			#&&
			!filter_var($submittedData[$this->fieldPrefix . 'NotificationEmail'], FILTER_VALIDATE_EMAIL)
		) {
			$schedulerModule->addMessage(
				$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.notificationEmail.validation.error'),
				t3lib_FlashMessage::ERROR
			);
			$validInput = false;
		}

		// At the end there has to be a slash
		if (
			empty($submittedData[$this->fieldPrefix . 'RemoteUrlMainPath'])
			||
			!filter_var($submittedData[$this->fieldPrefix . 'RemoteUrlMainPath'], FILTER_VALIDATE_URL)
			||
			substr($submittedData[$this->fieldPrefix . 'RemoteUrlMainPath'], -1, 1) != '/'
		) {
			$schedulerModule->addMessage(
				$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.remoteUrlMainPath.validation.error'),
				t3lib_FlashMessage::ERROR
			);
			$validInput = false;
		}
		
		// At the beginning no slash, at the end has to be a slash
		if (
			empty($submittedData[$this->fieldPrefix . 'LocalPath'])
			||
			!is_dir($_SERVER['DOCUMENT_ROOT'] . '/' . $submittedData[$this->fieldPrefix . 'LocalPath'])
			||
			substr($submittedData[$this->fieldPrefix . 'LocalPath'], -1, 1) != '/'
			||
			substr($submittedData[$this->fieldPrefix . 'LocalPath'], 0, 1) == '/'
		) {
			$schedulerModule->addMessage(
				$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.localPath.validation.error'),
				t3lib_FlashMessage::ERROR
			);
			$validInput = false;
		}
		
		// 'tx_pbfetchremotefiles' must be a part of the local file path (for reasons of safety)
		if(!preg_match('/tx_pbfetchremotefiles/', $submittedData[$this->fieldPrefix . 'LocalPath'])){
			$schedulerModule->addMessage(
				$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.localPath.validation.partNotAllowed.error'),
				t3lib_FlashMessage::ERROR
			);
		}
		
		
		/*
		// Not allowed parts in the local files
		$notAllowedInLocalPath = array('typo3', 't3lib', 'typo3conf', 'uploads');

		foreach($notAllowedInLocalPath as $notAllowed){
			if(preg_match('#' . $notAllowed . '#i', $submittedData[$this->fieldPrefix . 'LocalPath'])){
				$schedulerModule->addMessage(
					$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.localPath.validation.partNotAllowed.error')  . ' ' . implode(', ', $notAllowedInLocalPath),
					t3lib_FlashMessage::ERROR
				);
				$validInput = false;
			}
		}
		*/
		
		// Remote files has to start without a slash
		$remoteFiles	= explode("\n", $submittedData[$this->fieldPrefix . 'RemoteFiles']);
		$remoteFilesPathOk = true;
		$remoteFilesSlashOk = true;
		foreach($remoteFiles as $remoteFile){
			
			$remoteFile = chop($remoteFile);
			
/*
			foreach($notAllowedInLocalPath as $notAllowed){
				if(preg_match('#' . $notAllowed . '#i', $remoteFile)){
					$remoteFilesPathOk = false;
				}
			}
*/
			
			if(substr($remoteFile, 0, 1) == '/'){
				$remoteFilesSlashOk = false;
			}
			
		}
		
		if (
			empty($submittedData[$this->fieldPrefix . 'RemoteFiles'])
			#||
			#!$remoteFilesPathOk
			||
			!$remoteFilesSlashOk
		) {
			$schedulerModule->addMessage(
				$GLOBALS['LANG']->sL('LLL:EXT:pb_fetch_remote_files/locallang.xml:tasks.fetchfiles.remoteFiles.validation.error')  . ' ' . implode(', ', $notAllowedInLocalPath),
				t3lib_FlashMessage::ERROR
			);
			$validInput = false;
		}		
		
		return $validInput;

	}

	/**
	 * Takes care of saving the additional fields' values in the task's object
	 *
	 * @param	array	$submittedData An array containing the data submitted by the add/edit task form
	 * @param	tx_scheduler_Task	$task Reference to the scheduler backend module
	 * @return	void
	 */
	public function saveAdditionalFields(array $submittedData, tx_scheduler_Task $task) {

		/*
		if (!($task instanceof tx_reports_tasks_SystemStatusUpdateTask)) {
			throw new InvalidArgumentException(
				'Expected a task of type tx_reports_tasks_SystemStatusUpdateTask, but got ' . get_class($task),
				1295012802
			);
		}
		*/

		$task->setNotificationEmail($submittedData[$this->fieldPrefix . 'NotificationEmail']);
		$task->setMailMode($submittedData[$this->fieldPrefix . 'MailMode']);
		$task->setRemoteUrlMainPath($submittedData[$this->fieldPrefix . 'RemoteUrlMainPath']);
		$task->setRemoteFiles($submittedData[$this->fieldPrefix . 'RemoteFiles']);
		$task->setLocalPath($submittedData[$this->fieldPrefix . 'LocalPath']);

	}

	/**
	 * Constructs the full field name which can be used in HTML markup.
	 *
	 * @param	string	$fieldName A raw field name
	 * @return	string Field name ready to use in HTML markup
	 */
	protected function getFullFieldName($fieldName) {
		return $this->fieldPrefix . ucfirst($fieldName);
	}


}

if (defined('TYPO3_MODE') && isset($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/pb_fetch_remote_files/lib/scheduler/class.tx_pbfetchremotefiles_tasks_fetchfiles_additionalfieldprovider.php'])) {
	include_once($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/pb_fetch_remote_files/lib/scheduler/class.tx_pbfetchremotefiles_tasks_fetchfiles_additionalfieldprovider.php']);
}

?>